@@ -1,9 +1,8 @@
-use polar_core::parser;
-use polar_core::{events::*, kb::Bindings, polar::Polar, query::Query};
+use polar_core::{events::*, kb::Bindings, parser, polar::Polar, query::Query};
 
 pub fn runner_from_query(q: &str) -> Runner {
     let polar = Polar::new();
-    let query_term = parser::parse_query(0, q).unwrap();
+    let query_term = parser::parse_query(q).unwrap();
     let query = polar.new_query_from_term(query_term, false);
     Runner::new(polar, query)
 }
