@@ -5,7 +5,7 @@ use std::{
 
 use crate::{
     counter::Counter,
-    error::{invalid_state_error, RuntimeError},
+    error::{df_field_missing, df_unsupported_op, invalid_state, PolarResult},
     events::ResultEvent,
     filter::singleton,
     terms::*,
@@ -13,7 +13,6 @@ use crate::{
 
 use serde::{Deserialize, Serialize};
 
-type Result<A> = core::result::Result<A, RuntimeError>;
 type Id = u64;
 type VarId = Id;
 type TypeName = String;
@@ -129,7 +128,7 @@ pub fn build_filter_plan(
     partial_results: PartialResults,
     variable: &str,
     class_tag: &str,
-) -> Result<FilterPlan> {
+) -> PolarResult<FilterPlan> {
     FilterPlan::build(types, partial_results, variable, class_tag)
 }
 
@@ -144,7 +143,7 @@ impl From<Term> for Constraint {
 }
 
 impl VarInfo {
-    fn from_op(op: &Operation) -> Result<Self> {
+    fn from_op(op: &Operation) -> PolarResult<Self> {
         Self::default().process_exp(op)
     }
 
@@ -175,7 +174,7 @@ impl VarInfo {
     fn dot_var(&mut self, base: &Term, field: &Term) -> VarName {
         // handle nested dot ops.
         let sym = self.symbolize(base);
-        let field_str = field.value().as_string().unwrap();
+        let field_str = field.as_string().unwrap();
 
         match self
             .field_relationships
@@ -202,136 +201,112 @@ impl VarInfo {
 
     /// turn dot expressions into symbols but leave other things unchanged.
     fn undot(&mut self, term: &Term) -> Value {
-        let val = term.value();
-        match val.as_expression() {
+        match term.as_expression() {
             Ok(Operation {
                 operator: Operator::Dot,
                 args,
             }) if args.len() == 2 => Value::from(self.dot_var(&args[0], &args[1])),
-            _ => val.clone(),
+            _ => term.value().clone(),
         }
     }
 
-    fn do_and(self, args: &[Term]) -> Result<Self> {
+    fn do_and(self, args: &[Term]) -> PolarResult<Self> {
         args.iter().fold(Ok(self), |this, arg| {
-            this?.process_exp(arg.value().as_expression().unwrap())
+            this?.process_exp(arg.as_expression()?)
         })
     }
 
-    fn do_dot(mut self, lhs: &Term, rhs: &Term) -> Result<Self> {
+    fn do_dot(mut self, lhs: &Term, rhs: &Term) -> Self {
         self.dot_var(lhs, rhs);
-        Ok(self)
+        self
     }
 
-    fn do_isa(mut self, lhs: &Term, rhs: &Term) -> Result<Self> {
-        match rhs.value().as_pattern() {
+    fn do_isa(mut self, lhs: &Term, rhs: &Term) -> PolarResult<Self> {
+        match rhs.as_pattern() {
             Ok(Pattern::Instance(i)) if i.fields.fields.is_empty() => {
                 let lhs = self.symbolize(lhs);
                 self.types.push((lhs, i.tag.0.clone()));
                 Ok(self)
             }
-            _ => unsupported_op_error(Operation {
+            _ => df_unsupported_op(Operation {
                 operator: Operator::Isa,
                 args: vec![lhs.clone(), rhs.clone()],
             }),
         }
     }
 
-    fn do_unify(mut self, left: &Term, right: &Term) -> Result<Self> {
+    fn do_unify(mut self, left: &Term, right: &Term) -> PolarResult<Self> {
         match (self.undot(left), self.undot(right)) {
-            (Value::Variable(l), Value::Variable(r)) => {
-                self.cycles.push((l, r));
-                Ok(self)
-            }
+            (Value::Variable(l), Value::Variable(r)) => self.cycles.push((l, r)),
             (Value::Variable(var), val) | (val, Value::Variable(var)) => {
-                self.eq_values.push((var, Term::from(val)));
-                Ok(self)
+                self.eq_values.push((var, Term::from(val)))
             }
             // Unifying something else.
             // 1 = 1 is irrelevant for data filtering, other stuff seems like an error.
             // @NOTE(steve): Going with the same not yet supported message but if this is
             // coming through it's probably a bug in the simplifier.
-            _ => unsupported_op_error(Operation {
+            _ => df_unsupported_op(Operation {
                 operator: Operator::Unify,
                 args: vec![left.clone(), right.clone()],
-            }),
+            })?,
         }
+        Ok(self)
     }
 
-    fn do_neq(mut self, left: &Term, right: &Term) -> Result<Self> {
+    fn do_neq(mut self, left: &Term, right: &Term) -> PolarResult<Self> {
         match (self.undot(left), self.undot(right)) {
-            (Value::Variable(l), Value::Variable(r)) => {
-                self.uncycles.push((l, r));
-                Ok(self)
-            }
+            (Value::Variable(l), Value::Variable(r)) => self.uncycles.push((l, r)),
             (Value::Variable(l), _) => {
                 let r = self.symbolize(right);
-                self.uncycles.push((l, r));
-                Ok(self)
+                self.uncycles.push((l, r))
             }
             (_, Value::Variable(r)) => {
                 let l = self.symbolize(left);
-                self.uncycles.push((l, r));
-                Ok(self)
+                self.uncycles.push((l, r))
             }
-            _ => unsupported_op_error(Operation {
+            _ => df_unsupported_op(Operation {
                 operator: Operator::Neq,
                 args: vec![left.clone(), right.clone()],
-            }),
+            })?,
         }
+        Ok(self)
     }
 
-    fn do_in(mut self, left: &Term, right: &Term) -> Result<Self> {
+    fn do_in(mut self, left: &Term, right: &Term) -> PolarResult<Self> {
         match (self.undot(left), self.undot(right)) {
-            (Value::Variable(l), Value::Variable(r)) => {
-                self.in_relationships.push((l, r));
-                Ok(self)
-            }
-            (val, Value::Variable(var)) => {
-                self.contained_values.push((Term::from(val), var));
-                Ok(self)
-            }
-            _ => unsupported_op_error(Operation {
+            (Value::Variable(l), Value::Variable(r)) => self.in_relationships.push((l, r)),
+            (val, Value::Variable(var)) => self.contained_values.push((Term::from(val), var)),
+            _ => df_unsupported_op(Operation {
                 operator: Operator::In,
                 args: vec![left.clone(), right.clone()],
-            }),
+            })?,
         }
+        Ok(self)
     }
 
     /// Process an expression in the context of this VarInfo. Just does side effects.
-    fn process_exp(self, exp: &Operation) -> Result<Self> {
+    fn process_exp(self, exp: &Operation) -> PolarResult<Self> {
         use Operator::*;
         let args = &exp.args;
         match exp.operator {
             And => self.do_and(args),
-            Dot if args.len() == 2 => self.do_dot(&args[0], &args[1]),
+            Dot if args.len() == 2 => Ok(self.do_dot(&args[0], &args[1])),
             Isa if args.len() == 2 => self.do_isa(&args[0], &args[1]),
             Neq if args.len() == 2 => self.do_neq(&args[0], &args[1]),
             In if args.len() == 2 => self.do_in(&args[0], &args[1]),
             Unify | Eq | Assign if args.len() == 2 => self.do_unify(&args[0], &args[1]),
-            _ => unsupported_op_error(exp.clone()),
+            _ => df_unsupported_op(exp.clone()),
         }
     }
 }
 
-pub fn unsupported_op_error<A>(operation: Operation) -> Result<A> {
-    Err(RuntimeError::DataFilteringUnsupportedOp { operation })
-}
-
-pub fn unregistered_field_error<A>(var_type: &str, field: &str) -> Result<A> {
-    Err(RuntimeError::DataFilteringFieldMissing {
-        var_type: var_type.to_string(),
-        field: field.to_string(),
-    })
-}
-
 impl FilterPlan {
     fn build(
         types: Types,
         partial_results: PartialResults,
         var: &str,
         class_tag: &str,
-    ) -> Result<FilterPlan> {
+    ) -> PolarResult<FilterPlan> {
         // @NOTE(steve): Just reading an env var here sucks (see all the stuff we had to do
         // to get POLAR_LOG to work in all libs, wasm etc...) but that's what I'm doing today.
         // At some point surface this info better.
@@ -348,22 +323,23 @@ impl FilterPlan {
             // if the result doesn't include a binding for this variable,
             // or if the binding isn't an expression, then just ignore it.
             .filter_map(|(i, result)| {
-                result.bindings.get(&Symbol::new(var)).map(|term| {
-                    match term.value().as_expression() {
+                result
+                    .bindings
+                    .get(&Symbol::new(var))
+                    .map(|term| match term.as_expression() {
                         Ok(exp) if exp.operator == Operator::And => {
                             let vars = Vars::from_op(exp)?;
                             if explain {
-                                eprintln!("  {}: {}", i, term.to_polar());
+                                eprintln!("  {}: {}", i, term);
                                 vars.explain()
                             }
 
                             ResultSet::build(&types, &vars, class_tag)
                         }
                         _ => Ok(ResultSet::immediate(term.clone(), class_tag)),
-                    }
-                })
+                    })
             })
-            .collect::<Result<Vec<ResultSet>>>()?;
+            .collect::<PolarResult<Vec<_>>>()?;
 
         Ok(FilterPlan { result_sets }.optimize(explain))
     }
@@ -428,7 +404,7 @@ impl FilterPlan {
                     };
                     let field = &constraint.field;
                     let value = match &constraint.value {
-                        ConstraintValue::Term(t) => t.to_polar(),
+                        ConstraintValue::Term(t) => t.to_string(),
                         ConstraintValue::Field(f) => format!("FIELD({})", f),
                         ConstraintValue::Ref(r) => {
                             let inside = match &r.field {
@@ -463,7 +439,7 @@ impl ResultSet {
         }
     }
 
-    fn build(types: &Types, vars: &Vars, this_type: &str) -> Result<Self> {
+    fn build(types: &Types, vars: &Vars, this_type: &str) -> PolarResult<Self> {
         let result_set = ResultSet {
             requests: HashMap::new(),
             resolve_order: vec![],
@@ -507,7 +483,7 @@ impl FetchRequest {
 }
 
 impl<'a> ResultSetBuilder<'a> {
-    fn into_result_set(self) -> Result<ResultSet> {
+    fn into_result_set(self) -> PolarResult<ResultSet> {
         let mut rset = self.result_set;
         for (i, rid1) in rset.resolve_order.iter().enumerate() {
             let ro = &rset.resolve_order;
@@ -526,13 +502,13 @@ impl<'a> ResultSetBuilder<'a> {
 
         // error messages
         let missing = |id| {
-            invalid_state_error(format!(
+            invalid_state(format!(
                 "Request {} missing from resolve order {:?}",
                 id, order
             ))
         };
         let bad_order = |id1, id2, rset| {
-            invalid_state_error(format!(
+            invalid_state(format!(
                 "Result set {} is resolved before its dependency {} in {:?}",
                 id1, id2, rset
             ))
@@ -555,7 +531,7 @@ impl<'a> ResultSetBuilder<'a> {
         Ok(rset)
     }
 
-    fn constrain_var(&mut self, id: Id, var_type: &str) -> Result<&mut Self> {
+    fn constrain_var(&mut self, id: Id, var_type: &str) -> PolarResult<&mut Self> {
         if self.seen.insert(id) {
             // add a fetch request
             self.result_set.requests.insert(
@@ -569,8 +545,8 @@ impl<'a> ResultSetBuilder<'a> {
             // apply constraints to this request, then add to resolve order
             self.constrain_fields(id, var_type)?
                 .constrain_in_vars(id, var_type)?
-                .constrain_eq_vars(id)?
-                .constrain_neq_vars(id)?
+                .constrain_eq_vars(id)
+                .constrain_neq_vars(id)
                 .result_set
                 .resolve_order
                 .push(id);
@@ -578,7 +554,7 @@ impl<'a> ResultSetBuilder<'a> {
         Ok(self)
     }
 
-    fn constrain_neq_vars(&mut self, id: Id) -> Result<&mut Self> {
+    fn constrain_neq_vars(&mut self, id: Id) -> &mut Self {
         let request = self.result_set.requests.get_mut(&id).unwrap();
         for v in self.vars.uncycles.get(&id).into_iter().flatten() {
             let (kind, value) = if let Some(val) = self.vars.eq_values.get(v) {
@@ -594,10 +570,10 @@ impl<'a> ResultSetBuilder<'a> {
             };
             request.constrain(kind, None, value)
         }
-        Ok(self)
+        self
     }
 
-    fn constrain_eq_vars(&mut self, id: Id) -> Result<&mut Self> {
+    fn constrain_eq_vars(&mut self, id: Id) -> &mut Self {
         if let Some(t) = self.vars.eq_values.get(&id) {
             self.result_set.requests.get_mut(&id).unwrap().constrain(
                 ConstraintKind::Eq,
@@ -605,10 +581,10 @@ impl<'a> ResultSetBuilder<'a> {
                 ConstraintValue::Term(t.clone()),
             );
         }
-        Ok(self)
+        self
     }
 
-    fn constrain_in_vars(&mut self, id: Id, var_type: &str) -> Result<&mut Self> {
+    fn constrain_in_vars(&mut self, id: Id, var_type: &str) -> PolarResult<&mut Self> {
         let mut req = self.result_set.requests.remove(&id).unwrap();
 
         // Constrain any vars that are `in` this var.
@@ -636,7 +612,7 @@ impl<'a> ResultSetBuilder<'a> {
         Ok(self)
     }
 
-    fn constrain_field_eq(&mut self, id: Id, field: &str, child: Id) -> Result<&mut Self> {
+    fn constrain_field_eq(&mut self, id: Id, field: &str, child: Id) -> &mut Self {
         if let Some(v) = self.vars.eq_values.get(&child) {
             self.result_set.requests.get_mut(&id).unwrap().constrain(
                 ConstraintKind::Eq,
@@ -644,10 +620,10 @@ impl<'a> ResultSetBuilder<'a> {
                 ConstraintValue::Term(v.clone()),
             );
         }
-        Ok(self)
+        self
     }
 
-    fn constrain_field_neq(&mut self, id: Id, field: &str, child: Id) -> Result<&mut Self> {
+    fn constrain_field_neq(&mut self, id: Id, field: &str, child: Id) -> &mut Self {
         let req = self.result_set.requests.get_mut(&id).unwrap();
         for other_id in self.vars.uncycles.get(&child).into_iter().flatten() {
             match (
@@ -674,10 +650,10 @@ impl<'a> ResultSetBuilder<'a> {
                 _ => (),
             }
         }
-        Ok(self)
+        self
     }
 
-    fn constrain_field_contained(&mut self, id: Id, field: &str, child: Id) -> Result<&mut Self> {
+    fn constrain_field_contained(&mut self, id: Id, field: &str, child: Id) -> &mut Self {
         let request = self.result_set.requests.get_mut(&id).unwrap();
         self.vars
             .contained_values
@@ -691,7 +667,7 @@ impl<'a> ResultSetBuilder<'a> {
                     ConstraintValue::Term(v.clone()),
                 )
             });
-        Ok(self)
+        self
     }
 
     fn constrain_field_others_with_same_parent(
@@ -699,7 +675,7 @@ impl<'a> ResultSetBuilder<'a> {
         id: Id,
         my_field: &str,
         my_child: Id,
-    ) -> Result<&mut Self> {
+    ) -> &mut Self {
         for (other_field, other_child) in self
             .vars
             .field_relationships
@@ -716,15 +692,10 @@ impl<'a> ResultSetBuilder<'a> {
                 ConstraintValue::Field(other_field.clone()),
             );
         }
-        Ok(self)
+        self
     }
 
-    fn constrain_field_others(
-        &mut self,
-        id: Id,
-        my_field: &str,
-        my_child: Id,
-    ) -> Result<&mut Self> {
+    fn constrain_field_others(&mut self, id: Id, my_field: &str, my_child: Id) -> &mut Self {
         for (other_parent, other_children) in self
             .vars
             .field_relationships
@@ -744,7 +715,7 @@ impl<'a> ResultSetBuilder<'a> {
                 )
             }
         }
-        Ok(self)
+        self
     }
 
     fn constrain_other_field(
@@ -773,7 +744,7 @@ impl<'a> ResultSetBuilder<'a> {
         other_class_tag: &str,
         my_field: &str,
         other_field: &str,
-    ) -> Result<&mut Self> {
+    ) -> PolarResult<&mut Self> {
         self.constrain_var(child, other_class_tag)?
             .result_set
             .requests
@@ -797,18 +768,17 @@ impl<'a> ResultSetBuilder<'a> {
         field: &str,
         child: Id,
         before: usize,
-    ) -> Result<&mut Self> {
+    ) -> PolarResult<&mut Self> {
         let after = self.result_set.requests.get(&id).unwrap().len();
         if before != after {
             Ok(self)
         } else {
-            invalid_state_error(format!(
+            invalid_state(format!(
                 "Unsupported field access: {}.{} = {}",
-                self.var_name(id)
-                    .unwrap_or_else(|| Symbol(format!("{}", id))),
+                self.var_name(id).unwrap_or_else(|| Symbol(id.to_string())),
                 field,
                 self.var_name(child)
-                    .unwrap_or_else(|| Symbol(format!("{}", child))),
+                    .unwrap_or_else(|| Symbol(child.to_string())),
             ))
         }
     }
@@ -822,13 +792,13 @@ impl<'a> ResultSetBuilder<'a> {
         })
     }
 
-    fn constrain_fields(&mut self, id: Id, var_type: &str) -> Result<&mut Self> {
+    fn constrain_fields(&mut self, id: Id, var_type: &str) -> PolarResult<&mut Self> {
         match self.vars.field_relationships.get(&id) {
             None => Ok(self),
             Some(fs) => fs.iter().fold(Ok(self), |this, (field, child)| {
                 let this = this?;
                 match this.types.get(var_type).and_then(|m| m.get(field)) {
-                    None => unregistered_field_error(var_type, field),
+                    None => df_field_missing(var_type, field),
                     Some(Type::Relation {
                         other_class_tag,
                         my_field,
@@ -839,11 +809,11 @@ impl<'a> ResultSetBuilder<'a> {
                     }
                     _ => {
                         let before = this.result_set.requests.get(&id).unwrap().len();
-                        this.constrain_field_eq(id, field, *child)?
-                            .constrain_field_neq(id, field, *child)?
-                            .constrain_field_contained(id, field, *child)?
-                            .constrain_field_others_with_same_parent(id, field, *child)?
-                            .constrain_field_others(id, field, *child)?
+                        this.constrain_field_eq(id, field, *child)
+                            .constrain_field_neq(id, field, *child)
+                            .constrain_field_contained(id, field, *child)
+                            .constrain_field_others_with_same_parent(id, field, *child)
+                            .constrain_field_others(id, field, *child)
                             .ensure_added_constraint(id, field, *child, before)
                     }
                 }
@@ -853,13 +823,13 @@ impl<'a> ResultSetBuilder<'a> {
 }
 
 impl Vars {
-    fn from_op(op: &Operation) -> Result<Self> {
+    fn from_op(op: &Operation) -> PolarResult<Self> {
         Self::from_info(VarInfo::from_op(op)?)
     }
 
     /// Collapses the var info that we obtained from walking the expressions.
     /// Track equivalence classes of variables and assign each one an id.
-    fn from_info(info: VarInfo) -> Result<Self> {
+    fn from_info(info: VarInfo) -> PolarResult<Self> {
         let counter = info.counter;
 
         // group the variables into equivalence classes.
@@ -913,7 +883,7 @@ impl Vars {
         });
 
         seek_var_id(&variables, &sym!("_this")).map_or_else(
-            || invalid_state_error("No `_this` variable".to_string()),
+            || invalid_state("No `_this` variable"),
             |this_id| {
                 Ok(Vars {
                     variables,
@@ -959,11 +929,11 @@ impl Vars {
             };
             eprintln!("          type: {}", type_tag);
             if let Some(val) = self.eq_values.get(id) {
-                eprintln!("          value: {}", val.to_polar());
+                eprintln!("          value: {}", val);
             }
             if let Some(values) = self.contained_values.get(id) {
                 for val in values {
-                    eprintln!("          value contains: {}", val.to_polar());
+                    eprintln!("          value contains: {}", val);
                 }
             }
         }
@@ -1028,9 +998,10 @@ where
 #[cfg(test)]
 mod test {
     use super::*;
-    use crate::{bindings::Bindings, error::RuntimeError::*};
-
-    type TestResult = Result<()>;
+    use crate::{
+        bindings::Bindings,
+        error::{ErrorKind, RuntimeError::*},
+    };
 
     impl From<Bindings> for ResultEvent {
         fn from(bindings: Bindings) -> Self {
@@ -1075,7 +1046,7 @@ mod test {
     }
 
     #[test]
-    fn test_dot_plan() -> TestResult {
+    fn test_dot_plan() {
         let partial = test_input_0();
         let bindings = ResultEvent::from(hashmap! {
             sym!("resource") => partial
@@ -1093,23 +1064,21 @@ mod test {
                 }
             }
         };
-        build_filter_plan(types, vec![bindings], "resource", "A")?;
-        Ok(())
+        build_filter_plan(types, vec![bindings], "resource", "A").unwrap();
     }
 
     #[test]
-    fn test_empty_in() -> TestResult {
+    fn test_empty_in() {
         let partial = term!(op!(And, term!(op!(In, var!("_this"), var!("x")))));
         let bindings = ResultEvent::from(hashmap! {
             sym!("resource") => partial
         });
 
-        build_filter_plan(hashmap! {}, vec![bindings], "resource", "SomeClass")?;
-        Ok(())
+        build_filter_plan(hashmap! {}, vec![bindings], "resource", "SomeClass").unwrap();
     }
 
     #[test]
-    fn test_unregistered_field() -> TestResult {
+    fn test_unregistered_field() {
         let pat_a = term!(pattern!(instance!("A")));
         let partial = term!(op!(
             And,
@@ -1129,12 +1098,11 @@ mod test {
         };
 
         let err = build_filter_plan(types, vec![bindings], "resource", "A").unwrap_err();
-        match err {
-            RuntimeError::DataFilteringFieldMissing { var_type, field }
+        match err.0 {
+            ErrorKind::Runtime(DataFilteringFieldMissing { var_type, field })
                 if var_type == "A" && field == "field" => {}
             _ => panic!("unexpected {:?}", err),
         }
-        Ok(())
     }
 
     #[test]
@@ -1145,7 +1113,7 @@ mod test {
     }
 
     #[test]
-    fn test_dot_var_cycles() -> Result<()> {
+    fn test_dot_var_cycles() -> PolarResult<()> {
         let dot_op: Term = term!(op!(Dot, var!("x"), str!("y")));
         let op = op!(
             And,
@@ -1165,14 +1133,14 @@ mod test {
     #[test]
     fn test_unsupported_op_msgs() {
         let err = Vars::from_op(&op!(Dot)).expect_err("should've failed");
-        match err {
-            DataFilteringUnsupportedOp {
+        match err.0 {
+            ErrorKind::Runtime(DataFilteringUnsupportedOp {
                 operation:
                     Operation {
                         operator: Operator::Dot,
                         args,
                     },
-            } if args.is_empty() => (),
+            }) if args.is_empty() => (),
             _ => panic!("unexpected"),
         }
     }
