@@ -16,9 +16,7 @@ pub fn invert_operation(Operation { operator, args }: Operation) -> Operation {
     fn invert_args(args: Vec<Term>) -> Vec<Term> {
         args.into_iter()
             .map(|t| {
-                t.clone_with_value(value!(invert_operation(
-                    t.value().as_expression().unwrap().clone()
-                )))
+                t.clone_with_value(value!(invert_operation(t.as_expression().unwrap().clone())))
             })
             .collect()
     }
@@ -58,11 +56,7 @@ pub fn invert_operation(Operation { operator, args }: Operation) -> Operation {
         Leq => Operation { operator: Gt, args },
 
         // double negative
-        Not => args[0]
-            .value()
-            .as_expression()
-            .expect("negated expression")
-            .clone(),
+        Not => args[0].as_expression().expect("negated expression").clone(),
 
         // preserve the not
         Isa | In => Operation {
@@ -241,7 +235,7 @@ impl Operation {
     pub fn constraints(&self) -> Vec<Operation> {
         self.args
             .iter()
-            .map(|a| a.value().as_expression().unwrap().clone())
+            .map(|a| a.as_expression().unwrap().clone())
             .collect()
     }
 
@@ -302,9 +296,8 @@ mod test {
     use super::*;
 
     use crate::bindings::Bindings;
-    use crate::error::{ErrorKind, PolarError, RuntimeError};
+    use crate::error::{ErrorKind, PolarError, PolarResult, RuntimeError};
     use crate::events::QueryEvent;
-    use crate::formatting::ToPolarString;
     use crate::polar::Polar;
     use crate::query::Query;
     use crate::terms::{Call, Dictionary, InstanceLiteral, Pattern};
@@ -315,10 +308,9 @@ mod test {
                 $bindings
                     .get(&sym!($sym))
                     .expect(&format!("{} is unbound", $sym))
-                    .value()
                     .as_expression()
                     .unwrap()
-                    .to_polar(),
+                    .to_string(),
                 $right
             )
         };
@@ -328,14 +320,13 @@ mod test {
             let l = $bindings
                     .get(&sym!($sym))
                     .expect(&format!("{} is unbound", $sym))
-                    .value()
                     .as_expression()
                     .unwrap()
                     .clone()
                     .into_iter()
                     .collect::<HashSet<Term>>();
             let r = hashset! { $($args),+ };
-            let fmt = |hs: &HashSet<Term>| format!("{{ {} }}", hs.iter().map(|t| t.to_polar()).collect::<Vec<String>>().join(", "));
+            let fmt = |hs: &HashSet<Term>| format!("{{ {} }}", hs.iter().map(Term::to_string).collect::<Vec<_>>().join(", "));
 
             assert_eq!(&l, &r, "{} != {}", fmt(&l), fmt(&r));
 
@@ -362,8 +353,8 @@ mod test {
                         "Bindings: {}",
                         bindings
                             .iter()
-                            .map(|(k, v)| format!("{}: {}", k.0, v.to_polar()))
-                            .collect::<Vec<String>>()
+                            .map(|(k, v)| format!("{}: {}", k.0, v))
+                            .collect::<Vec<_>>()
                             .join("\n")
                     )
                 } else {
@@ -385,7 +376,7 @@ mod test {
         }};
     }
 
-    fn next_binding(query: &mut Query) -> Result<Bindings, PolarError> {
+    fn next_binding(query: &mut Query) -> PolarResult<Bindings> {
         let event = query.next_event()?;
         if let QueryEvent::Result { bindings, .. } = event {
             Ok(bindings)
@@ -394,7 +385,7 @@ mod test {
         }
     }
 
-    type TestResult = Result<(), PolarError>;
+    type TestResult = PolarResult<()>;
 
     #[test]
     fn basic_test() -> TestResult {
@@ -1007,11 +998,8 @@ mod test {
         let mut q = p.new_query_from_term(term!(call!("g", [sym!("a")])), false);
         let error = q.next_event().unwrap_err();
         assert!(matches!(
-            error,
-            PolarError {
-                kind: ErrorKind::Runtime(RuntimeError::Unsupported { .. }),
-                ..
-            }
+            error.0,
+            ErrorKind::Runtime(RuntimeError::Unsupported { .. }),
         ));
         Ok(())
     }
@@ -1064,7 +1052,7 @@ mod test {
                         let last_segment = path.last().unwrap();
                         q.question_result(
                             call_id,
-                            last_segment.value().as_string().unwrap().to_uppercase() == class_tag.0,
+                            last_segment.as_string().unwrap().to_uppercase() == class_tag.0,
                         )
                         .unwrap();
                     }
@@ -1765,11 +1753,8 @@ mod test {
         let mut q = p.new_query_from_term(term!(call!("f", [sym!("x")])), false);
         let error = q.next_event().unwrap_err();
         assert!(matches!(
-            error,
-            PolarError {
-                kind: ErrorKind::Runtime(RuntimeError::Unsupported { .. }),
-                ..
-            }
+            error.0,
+            ErrorKind::Runtime(RuntimeError::Unsupported { .. }),
         ));
         Ok(())
     }
@@ -2348,14 +2333,13 @@ mod test {
             assert!(
                 matches!(
                     res,
-                    Err(PolarError {
-                        kind: ErrorKind::Runtime(RuntimeError::UnhandledPartial { .. }),
-                        ..
-                    })
+                    Err(PolarError(ErrorKind::Runtime(
+                        RuntimeError::UnhandledPartial { .. }
+                    )))
                 ),
                 "unexpected result: {:#?} for {}",
                 res,
-                query.to_polar()
+                query
             );
         }
 
