@@ -1,20 +1,18 @@
-use std::collections::hash_map::DefaultHasher;
-use std::collections::{BTreeMap, HashSet};
-use std::fmt;
-use std::hash::{Hash, Hasher};
-use std::sync::Arc;
+use std::{
+    collections::{hash_map::DefaultHasher, BTreeMap, HashSet},
+    fmt,
+    hash::{Hash, Hasher},
+    sync::Arc,
+};
 
 use serde::{Deserialize, Serialize};
 
-use super::error::RuntimeError::{self, InvalidState};
-pub use super::formatting::ToPolarString;
+use super::error::{unexpected_value, PolarResult};
 pub use super::numerics::Numeric;
 use super::resource_block::{ACTOR_UNION_NAME, RESOURCE_UNION_NAME};
-use super::sources::SourceInfo;
+use super::sources::{Context, Source, SourceInfo};
 use super::visitor::{walk_operation, walk_term, Visitor};
 
-type Result<T> = core::result::Result<T, RuntimeError>;
-
 #[derive(Debug, Clone, Serialize, Deserialize, Default, Eq, PartialEq, Hash)]
 pub struct Dictionary {
     pub fields: BTreeMap<Symbol, Term>,
@@ -44,15 +42,7 @@ pub struct ExternalInstance {
     pub constructor: Option<Term>,
     pub repr: Option<String>,
     pub class_repr: Option<String>,
-}
-
-// Context stored somewhere by id.
-
-// parser outputs
-
-#[derive(Debug, Clone, Serialize, Deserialize)]
-pub struct Context {
-    pub file: String,
+    pub class_id: Option<u64>,
 }
 
 pub type TermList = Vec<Term>;
@@ -149,70 +139,6 @@ pub enum Value {
 }
 
 impl Value {
-    pub fn as_symbol(&self) -> Result<&Symbol> {
-        match self {
-            Value::Variable(name) => Ok(name),
-            Value::RestVariable(name) => Ok(name),
-            _ => Err(InvalidState {
-                msg: format!("Expected symbol, got: {}", self),
-            }),
-        }
-    }
-
-    pub fn as_string(&self) -> Result<&str> {
-        match self {
-            Value::String(string) => Ok(string.as_ref()),
-            _ => Err(InvalidState {
-                msg: format!("Expected string, got: {}", self),
-            }),
-        }
-    }
-
-    pub fn as_expression(&self) -> Result<&Operation> {
-        match self {
-            Value::Expression(op) => Ok(op),
-            _ => Err(InvalidState {
-                msg: format!("Expected expression, got: {}", self),
-            }),
-        }
-    }
-
-    pub fn as_call(&self) -> Result<&Call> {
-        match self {
-            Value::Call(pred) => Ok(pred),
-            _ => Err(InvalidState {
-                msg: format!("Expected call, got: {}", self),
-            }),
-        }
-    }
-
-    pub fn as_pattern(&self) -> Result<&Pattern> {
-        match self {
-            Value::Pattern(p) => Ok(p),
-            _ => Err(InvalidState {
-                msg: format!("Expected pattern, got: {}", self),
-            }),
-        }
-    }
-
-    pub fn as_list(&self) -> Result<&TermList> {
-        match self {
-            Value::List(l) => Ok(l),
-            _ => Err(InvalidState {
-                msg: format!("Expected list, got: {}", self),
-            }),
-        }
-    }
-
-    pub fn as_dict(&self) -> Result<&Dictionary> {
-        match self {
-            Value::Dictionary(d) => Ok(d),
-            _ => Err(InvalidState {
-                msg: format!("Expected dictionary, got: {}", self),
-            }),
-        }
-    }
-
     pub fn is_ground(&self) -> bool {
         match self {
             Value::Call(_)
@@ -361,13 +287,9 @@ impl Term {
     }
 
     /// Creates a new term from the parser
-    pub fn new_from_parser(src_id: u64, left: usize, right: usize, value: Value) -> Self {
+    pub fn new_from_parser(source: Arc<Source>, left: usize, right: usize, value: Value) -> Self {
         Self {
-            source_info: SourceInfo::Parser {
-                src_id,
-                left,
-                right,
-            },
+            source_info: SourceInfo::parser(source, left, right),
             value: Arc::new(value),
         }
     }
@@ -394,17 +316,15 @@ impl Term {
         self.value = Arc::new(value);
     }
 
-    pub fn offset(&self) -> usize {
-        if let SourceInfo::Parser { left, .. } = self.source_info {
-            left
-        } else {
-            0
-        }
+    pub(crate) fn source_info(&self) -> &SourceInfo {
+        &self.source_info
     }
 
-    pub fn span(&self) -> Option<(usize, usize)> {
-        if let SourceInfo::Parser { left, right, .. } = self.source_info {
-            Some((left, right))
+    // TODO(gj): Parsed<T> type (or something) so we can remove this meaningless distinction
+    // between terms & rules.
+    pub(crate) fn parsed_context(&self) -> Option<&Context> {
+        if let SourceInfo::Parser(context) = self.source_info() {
+            Some(context)
         } else {
             None
         }
@@ -415,6 +335,59 @@ impl Term {
         &self.value
     }
 
+    pub(crate) fn as_symbol(&self) -> PolarResult<&Symbol> {
+        match self.value() {
+            Value::Variable(name) => Ok(name),
+            Value::RestVariable(name) => Ok(name),
+            _ => unexpected_value("(rest) variable", self.clone()),
+        }
+    }
+
+    pub(crate) fn as_string(&self) -> PolarResult<&str> {
+        match self.value() {
+            Value::String(string) => Ok(string.as_ref()),
+            _ => unexpected_value("string", self.clone()),
+        }
+    }
+
+    // Can't currently be `pub(crate)` due to use in oso crate.
+    pub fn as_expression(&self) -> PolarResult<&Operation> {
+        match self.value() {
+            Value::Expression(op) => Ok(op),
+            _ => unexpected_value("expression", self.clone()),
+        }
+    }
+
+    pub(crate) fn as_call(&self) -> PolarResult<&Call> {
+        match self.value() {
+            Value::Call(pred) => Ok(pred),
+            _ => unexpected_value("call", self.clone()),
+        }
+    }
+
+    pub(crate) fn as_pattern(&self) -> PolarResult<&Pattern> {
+        match self.value() {
+            Value::Pattern(p) => Ok(p),
+            _ => unexpected_value("pattern", self.clone()),
+        }
+    }
+
+    // Can't currently be `pub(crate)` due to use in polar-language-server crate.
+    pub fn as_list(&self) -> PolarResult<&TermList> {
+        match self.value() {
+            Value::List(l) => Ok(l),
+            _ => unexpected_value("list", self.clone()),
+        }
+    }
+
+    // Can't currently be `pub(crate)` due to use in polar-language-server crate.
+    pub fn as_dict(&self) -> PolarResult<&Dictionary> {
+        match self.value() {
+            Value::Dictionary(d) => Ok(d),
+            _ => unexpected_value("dictionary", self.clone()),
+        }
+    }
+
     /// Get a mutable reference to the underlying data.
     /// This will be a real mut pointer if there is only one
     /// term with an Arc to the value, otherwise it will be
@@ -488,14 +461,6 @@ impl Term {
         hasher.finish()
     }
 
-    pub fn get_source_id(&self) -> Option<u64> {
-        if let SourceInfo::Parser { src_id, .. } = self.source_info {
-            Some(src_id)
-        } else {
-            None
-        }
-    }
-
     pub fn is_actor_union(&self) -> bool {
         matches!(self.value(), Value::Pattern(Pattern::Instance(InstanceLiteral { tag, .. })) | Value::Variable(tag) if tag.0 == ACTOR_UNION_NAME)
     }
