@@ -1,8 +1,8 @@
 #[cfg(test)]
 mod tests {
-    use std::collections::BTreeMap;
+    use std::{collections::BTreeMap, sync::Arc};
 
-    use polar_core::{error::*, events::*, rules::*, term, terms::*, value};
+    use polar_core::{error::*, events::*, rules::*, sources::Source, term, terms::*, value};
 
     #[test]
     fn serialize_test() {
@@ -54,6 +54,7 @@ mod tests {
             constructor: None,
             repr: None,
             class_repr: None,
+            class_id: None,
         }));
         let list_of = Term::new_from_test(Value::List(vec![external]));
         eprintln!("{}", serde_json::to_string(&list_of).unwrap());
@@ -61,15 +62,16 @@ mod tests {
         fields.insert(Symbol::new("foo"), list_of);
         let dict = Term::new_from_test(Value::Dictionary(Dictionary { fields }));
         eprintln!("{}", serde_json::to_string(&dict).unwrap());
-        let e = ParseError::InvalidTokenCharacter {
+        let kind = ParseErrorKind::InvalidTokenCharacter {
             token: "Integer".to_owned(),
             c: 'x',
-            loc: 99,
+            loc: 15,
         };
-        let err = PolarError {
-            kind: ErrorKind::Parse(e),
-            context: None,
+        let e = ParseError {
+            kind,
+            source: Arc::new(Source::new("CinnabarVinegarInteger")),
         };
+        let err = PolarError::from(e);
         eprintln!("{}", serde_json::to_string(&err).unwrap());
         let rule = Rule::new_from_test(
             Symbol::new("foo"),
